import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0xffffff);
        this._scene = scene;

        this._clock = new THREE.Clock();

        this._setupCamera();
        //this._setupHelper();
        this._setupModel();
        //this._setupAnimation();
        this._setupGUI();
        this._setupLight();
        
        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(40, window.innerWidth / window.innerHeight, 1, 15000);
        camera.position.z = 3200;

        this._camera = camera;
    }

    _setupHelper() {}

    _setupModel() {
        const sphere = new THREE.Mesh(new THREE.SphereGeometry(100, 20, 20), new THREE.MeshNormalMaterial());
        this._scene.add(sphere);

        const geometry = new THREE.CylinderGeometry(0, 10, 100, 12);
        geometry.rotateX(Math.PI / 2);
        const material = new THREE.MeshNormalMaterial();

        for(let i=0; i<1000; i++) {
            const mesh = new THREE.Mesh(geometry, material);
            mesh.position.x = Math.random() * 4000 - 2000;
            mesh.position.y = Math.random() * 4000 - 2000;
            mesh.position.z = Math.random() * 4000 - 2000;
            mesh.scale.x = mesh.scale.y = mesh.scale.z = Math.random() * 4 + 2;
            this._scene.add(mesh);
        }

        this._sphere = sphere;
    }

    _setupAnimation() {}

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);

        this._mouseX = 0;
        this._mouseY = 0;
        document.addEventListener("mousemove", this._onDocumentMouseMove.bind(this));

        this._stats = stats;
    }

    _onDocumentMouseMove(event) {
        this._mouseX = (event.clientX - window.innerWidth / 2) * 10;
        this._mouseY = (event.clientY - window.innerHeight / 2) * 10;
    }

    _setupLight() {
        const light = new THREE.DirectionalLight(0xffffff, 2);
        light.position.set(1, 1, 1);
        this._scene.add(light);
    }

    update() {
        const time = Date.now() * 0.0005;
        this._sphere.position.x = Math.sin(time * 0.7) * 2000;
        this._sphere.position.y = Math.cos(time * 0.5) * 2000;
        this._sphere.position.z = Math.cos(time * 0.3) * 2000;

        for(let i=1, l=this._scene.children.length; i<l; i++) {
            this._scene.children[i].lookAt(this._sphere.position);
        }

        this._camera.position.x += (this._mouseX - this._camera.position.x) * 0.05;
        this._camera.position.y += (-this._mouseY - this._camera.position.y) * 0.05;
        this._camera.lookAt(this._scene.position);

        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this._camera.aspect = aspect;
        this._camera.updateProjectionMatrix();

        this._renderer.setPixelRatio( window.devicePixelRatio );
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    (new App()).render();
}